var Cc = Components.classes;
var Ci = Components.interfaces;

const CERTS_DIR = "chrome://infonotary-mozilla-setup/content/certs/";

const nsX509CertDB = "@mozilla.org/security/x509certdb;1";
const nsIOService = "@mozilla.org/network/io-service;1"
const nsScriptableInputStream = "@mozilla.org/scriptableinputstream;1"

const RootCerts = new Array( "00-root-ca.b64" );

const CACerts = new Array(
	"01-qsign-ca.b64",
	"02-qsign-company-ca.b64",
	"03-qsign-personal-ca.b64",
	"04-qsign-finlmt-ca.b64",
	"05-trustpath-ca.b64",
	"06-domain-ca.b64",
	"07-codesign-ca.b64",
	"08-identity-ca.b64",
	"09-secmail-ca.b64",
	"10-encription-ca.b64",
	"11-hosted-ca.b64",
	"12-validation-ca.b64"
	);

var certdb = Cc[nsX509CertDB].getService(Ci.nsIX509CertDB);
var certdb2 = Cc[nsX509CertDB].getService(Ci.nsIX509CertDB2);

function getContents(aURL)
{
	var ioService=Cc[nsIOService].getService(Ci.nsIIOService);
	var scriptableStream=Cc[nsScriptableInputStream].getService(Ci.nsIScriptableInputStream);

	var channel=ioService.newChannel(aURL,null,null);
	var input=channel.open();
	scriptableStream.init(input);
	var str=scriptableStream.read(input.available());
	scriptableStream.close();
	input.close();
	return str;
}

function install_cert( b64, isRoot )
{
	var flags = ",,";
	if(isRoot)
		flags = "CT,C,C";
	try{
		certdb2.addCertFromBase64(b64, flags, "");
	}catch (e) {
	}
}

function set_trust( cert )
{
	try{
		certdb.setCertTrust( cert, cert.CA_CERT,
			certdb.TRUSTED_SSL | certdb.TRUSTED_EMAIL | certdb.TRUSTED_OBJSIGN );
	}catch (e){
	}
}

function check_cert( b64, isRoot )
{
	var cert = certdb.constructX509FromBase64(b64);
	var dbCert = null;

	try	{
		dbCert = certdb.findCertByNickname(null, cert.nickname);
	}
	catch (e) {
	}
	if( dbCert )
	{
		if( isRoot )
			set_trust( dbCert );
	}
	else
		install_cert( b64, isRoot );
}

function check_trust( cert )
{
	var res = false;
	try{
		res = certdb.isCertTrusted( cert, cert.CA_CERT, certdb.TRUSTED_SSL ) &&
			certdb.isCertTrusted( cert, cert.CA_CERT, certdb.TRUSTED_EMAIL ) &&
			certdb.isCertTrusted( cert, cert.CA_CERT, certdb.TRUSTED_OBJSIGN );
	}catch (e){
	}
	return res;
		/*if( !certdb.isCertTrusted( cert, cert.CA_CERT, certdb.TRUSTED_SSL ) ) {
			certdb.setCertTrust( cert, cert.CA_CERT, certdb.TRUSTED_SSL );
		}
		if( !certdb.isCertTrusted( cert, cert.CA_CERT, certdb.TRUSTED_EMAIL ) ) {
			certdb.setCertTrust( cert, cert.CA_CERT, certdb.TRUSTED_EMAIL );
		}
		if( !certdb.isCertTrusted( cert, cert.CA_CERT, certdb.TRUSTED_OBJSIGN ) ) {
			certdb.setCertTrust( cert, cert.CA_CERT, certdb.TRUSTED_OBJSIGN );
		}*/
}

function cert_is_ok( b64, isRoot )
{
	var cert = certdb.constructX509FromBase64(b64);
	var dbCert = null;

	try	{
		dbCert = certdb.findCertByNickname(null, cert.nickname);
	}
	catch (e) {
	}
	if( dbCert )
	{
		if( isRoot )
			return check_trust( dbCert );
		return true
	}
	return false;
}

function cert_chain_is_installed()
{
	try {
		var x;
		for (x in RootCerts)
		{
			if( !cert_is_ok( getContents( CERTS_DIR + RootCerts[x] ), true ) )
				return false;
		}
		
		for (x in CACerts)
		{
			if( !cert_is_ok( getContents( CERTS_DIR + CACerts[x] ), false ) )
				return false;
		}
	}
	catch(e) {
	}
	return true;
}

function setup_cert_chain( gui )
{
	try {
		var x;
		for (x in RootCerts)
			check_cert( getContents( CERTS_DIR + RootCerts[x] ), true );
		
		for (x in CACerts)
			check_cert( getContents( CERTS_DIR + CACerts[x] ), false );
	}
	catch(e) {
	}
	if( gui ) {
		var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                        .getService(Components.interfaces.nsIPromptService);
		var strbundle = document.getElementById("strings");
		var title = strbundle.getString("inSetupPrefsTitle");
		var msg = strbundle.getString("certChainInstalled");
		prompts.alert(null, title, msg);
	}
	return true;
}