var Cc = Components.classes;
var Ci = Components.interfaces;

const Siemens = "Siemens PKCS#11 module";
const Charismathics = "Charismathics PKCS#11 module";
const OpenSC = "OpenSC PKCS#11 module";
const Bit4id = "Bit4id PKCS#11 module";

const FRIENDLY_FLAG = 0x1<<28

function is_silent_module_install_posible()
{
	return pkcs11 == null;
}

function PKCS11Module( aName, aPath )
{
	this.name = aName;
	this.path = aPath;
	this.m_nStatus = 0;
}

PKCS11Module.prototype.__defineGetter__("status",
	function() {
		return this.m_nStatus;
	}
);

PKCS11Module.prototype.isRegistred = function( mod_name )
{
	var moddb = Cc["@mozilla.org/security/pkcs11moduledb;1"].getService(Ci.nsIPKCS11ModuleDB);
	var enum = moddb.listModules();
	try{
		for( enum.first(); ; enum.next() ) {
			var mod = enum.currentItem().QueryInterface(Ci.nsIPKCS11Module);
			if( mod.libName == null )
				continue;
			if( mod.libName.toLowerCase() == this.path.toLowerCase() ) {
				if( mod_name ) {
					mod_name.value = mod.name;
				}
				return true;
			}
		}
	}catch(err){
	}
	return false;
}

PKCS11Module.prototype.unregister = function(path)
{
	var tmp_name = {};
	var tmp_obj = new PKCS11Module("Temp", path);
	if( !tmp_obj.isRegistred(tmp_name) )
		return;
	if( !is_silent_module_install_posible() ) {
		//Gecko version < 1.9.1
		pkcs11.deleteModule(tmp_name.value);
	}
	else {
		//Gecko version >= 1.9.1
		var nsPKCS11 = Cc["@mozilla.org/security/pkcs11;1"].getService(Ci.nsIPKCS11);
		try {
			nsPKCS11.deleteModule(tmp_name.value);
		}
		catch( ex ) {
		}
	}
}

PKCS11Module.prototype.register = function()
{
	if( !this.isRegistred() ) {
		if( !is_silent_module_install_posible() ) {
			//Gecko version < 1.9.1
			this.m_nStatus = pkcs11.addmodule( this.name, this.path, FRIENDLY_FLAG, 0 );
		}
		else {
			//Gecko version >= 1.9.1
			var nsPKCS11 = Cc["@mozilla.org/security/pkcs11;1"].getService(Ci.nsIPKCS11);
			try {
				nsPKCS11.addModule( this.name, this.path, FRIENDLY_FLAG, 0 );
				this.m_nStatus = 1;
			}
			catch( ex ) {
				this.m_nStatus = -100; // Registration failed with exception
			}
		}
		
		//If module is onepin-opensc-pkcs11 remove opensc-pkcs11
		if( this.m_nStatus > 0 ) {
			if( this.path.indexOf("onepin-opensc-pkcs11") != -1 ) {
				var temp_path = this.path.replace("onepin-opensc-pkcs11", "opensc-pkcs11");
				this.unregister(temp_path);
			}
		}
	}
	else
		this.m_nStatus = -51; // Module exists
	return this.m_nStatus;
}

function PKCS11Setup() {

	this.isWindows = false;
	this.isLinux = false;
	this.isMac = false;

	this.sysdir = "";
	this.known_folders = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties);
	try{
		this.sysdir = (this.known_folders.get("SysD", Ci.nsIFile)).path;
	}
	catch( err ) {
	}

	//this.moddb = Cc["@mozilla.org/security/pkcs11moduledb;1"].getService(Ci.nsIPKCS11ModuleDB);

	var runtime = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULRuntime);
	this.osString = runtime.OS;
	this.processor = runtime.XPCOMABI.split("-")[0];
	//WINNT is Windows 2000 and above, WIN95 is Windows 95, Windows 98 or Windows Me
	if( this.osString.indexOf("WIN")!=-1 )
		this.isWindows = true;
	else if( this.osString.toLowerCase().indexOf("linux")!=-1 )
		this.isLinux = true;
	else if( this.osString == "Darwin" )
		this.isMac = true;
	//Settup for BSD and Solaris is the same as settup for Linux
	else if( this.osString.indexOf("BSD")!=-1 )
		this.isLinux = true;
	else if( this.osString == "SunOS" )
		this.isLinux = true;
}

PKCS11Setup.prototype.__defineGetter__("isOK",
	function() {
		return this.isWindows ||
			this.isLinux ||
			this.isMac;
	}
);

PKCS11Setup.prototype.run_program = function( program, param, strToFind )
{
	// create an nsILocalFile for the shell
	var programFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
	programFile.initWithPath("/bin/sh");

	// create an nsIProcess
	var process = Cc["@mozilla.org/process/util;1"].createInstance(Ci.nsIProcess);
	process.init(programFile);

	var dirServeice
	var tempScript = this.known_folders.get("TmpD", Ci.nsIFile);
	var tempOutput = this.known_folders.get("TmpD", Ci.nsIFile);
	tempOutput.append("libtest.result");
	tempOutput.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, 0600);

	tempScript.append("libtest.sh");
	tempScript.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, 0600);

	var script = 'TMP=`' + program + ' ' + param + ' | grep ' + strToFind + '`\n' +
			'if [ "$TMP" != "" ]; then\n' +
			'echo "1" > ' + tempOutput.path + '\n' +
			'else\n' +
			'echo "Not found" > ' + tempOutput.path + '\n' +
			'fi\n';

	// Create test script
	var foStream = Cc["@mozilla.org/network/file-output-stream;1"].
					createInstance(Ci.nsIFileOutputStream);
	foStream.init(tempScript, 0x02 | 0x08 | 0x20, 0600, 0); 
	var converter = Cc["@mozilla.org/intl/converter-output-stream;1"].
					createInstance(Ci.nsIConverterOutputStream);
	converter.init(foStream, "UTF-8", 0, 0);
	converter.writeString(script);
	converter.close(); // this closes foStream

	var args = [tempScript.path];
	process.run(true, args, args.length);

	var fstream = Cc["@mozilla.org/network/file-input-stream;1"].
					createInstance(Ci.nsIFileInputStream);
	var cstream = Cc["@mozilla.org/intl/converter-input-stream;1"].
					createInstance(Ci.nsIConverterInputStream);
	fstream.init(tempOutput, -1, 0, 0);
	cstream.init(fstream, "UTF-8", 0, 0);

	var str = {};
	cstream.readString(-1, str); // read the whole file and put it in str.value
	cstream.close(); // this closes fstream

	tempScript.remove(false);
	tempOutput.remove(false);
	
	return str.value.indexOf("1") >= 0;
}

PKCS11Setup.prototype.search_for_opensc_linux = function()
{
	var file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
	// Check for OpenSC
	file.initWithPath("/usr/lib/onepin-opensc-pkcs11.so");
	if( file.exists() )
		return new PKCS11Module( OpenSC, file.path );
	file.initWithPath("/usr/local/lib/onepin-opensc-pkcs11.so");
	if( file.exists() )
		return new PKCS11Module( OpenSC, file.path );
	file.initWithPath("/usr/lib/opensc-pkcs11.so");
	if( file.exists() )
		return new PKCS11Module( OpenSC, file.path );
	file.initWithPath("/usr/local/lib/opensc-pkcs11.so");
	if( file.exists() )
		return new PKCS11Module( OpenSC, file.path );
	if( this.processor == "x86_64" )
	{
		//If nothing is found and we run on x86_64 try lib64 directory
		file.initWithPath("/usr/lib64/onepin-opensc-pkcs11.so");
		if( file.exists() )
			return new PKCS11Module( OpenSC, file.path );
		file.initWithPath("/usr/lib64/opensc-pkcs11.so");
		if( file.exists() )
			return new PKCS11Module( OpenSC, file.path );
	}
	return null;
}

PKCS11Setup.prototype.search_for_pkcs11_module = function()
{
	var results = new Array();
	var arrCounter = 0;
	var file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
	if( this.isWindows ) {
		if( this.processor == "x86_64" ) {
			//Check for Charismathics 64 bit
			file.initWithPath(this.sysdir + "\\cmP1164.dll");
			if( file.exists() )
				results[arrCounter++] = new PKCS11Module( Charismathics, file.path );
		}
		else {
			//Check for Charismathics
			file.initWithPath(this.sysdir + "\\cmP11.dll");
			if( file.exists() )
				results[arrCounter++] = new PKCS11Module( Charismathics, file.path );
			else {
				//Check for Siemens
				file.initWithPath(this.sysdir + "\\siecap11.dll");
				if( file.exists() )
					results[arrCounter++] = new PKCS11Module( Siemens, file.path );
			}
		}
		//Check for Bit4id
		file.initWithPath(this.sysdir + "\\bit4ipki.dll");
		if( file.exists() )
			results[arrCounter++] = new PKCS11Module( Bit4id, file.path );
	}
	else if( this.isLinux ) {
		var opensc_module = this.search_for_opensc_linux();
		if( opensc_module != null )
			results[arrCounter++] = opensc_module;
		else if( this.processor == "x86" ) { // Check for Siemens only on x86
			file.initWithPath("/usr/local/lib/libsiecap11.so");
			if( file.exists() )
				results[arrCounter++] = new PKCS11Module( Siemens, file.path );
		}
		file.initWithPath("/usr/lib/libbit4ipki.so");
		if( file.exists() )
			results[arrCounter++] = new PKCS11Module( Bit4id, file.path );
		else if( this.processor == "x86_64" ) {
			file.initWithPath("/usr/lib64/libbit4ipki.so");
			if( file.exists() )
				results[arrCounter++] = new PKCS11Module( Bit4id, file.path );
		}
	}
	else if( this.isMac ) {
		var onepin_found = false;
		//onepin-opensc-pkcs11.so in SCA builds is built for PowerPC only,
		//but in OpenSC installers it is built for all arhitectures, so I must check file arch.
		file.initWithPath("/Library/OpenSC/lib/onepin-opensc-pkcs11.so");
		if( file.exists() ) {
			try{
				var proc_type = this.processor;
				if( proc_type == "x86" )
					proc_type = "i386";
				if( this.run_program("/usr/bin/file", file.path, proc_type ) ) {
					results[arrCounter++] = new PKCS11Module( OpenSC, file.path );
					onepin_found = true;
				}
			} catch(e){
            }
		}
		file.initWithPath("/Library/OpenSC/lib/opensc-pkcs11.so");
		if( file.exists() )
			if( !onepin_found )
				results[arrCounter++] = new PKCS11Module( OpenSC, file.path );
		file.initWithPath("/System/Library/bit4id/cryptoki/libbit4ipki.dylib");
		if( file.exists() )
			results[arrCounter++] = new PKCS11Module( Bit4id, file.path );
	}
	if( results.length > 0 )
		return results;
	return null;
}

// Return codes from https://developer.mozilla.org/en/PKCS11_Jar_Install
PKCS11Setup.prototype.setup = function()
{
	var res = this.search_for_pkcs11_module();
	if( res != null )
		for( var i = 0; i < res.length; ++i )
			res[i].register();
	return res;
}

function setup_pkcs11_module( gui )
{
	var prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"]
					.getService(Ci.nsIPromptService);
	var strbundle = document.getElementById("strings");
	var setup_res;
	var result = true;
	var obj = new PKCS11Setup();
	if( obj.isOK ) {
		setup_res = obj.setup();
	}
	else if( gui ) {
		var msg = strbundle.getFormattedString("osNotSupported", [ obj.osString ]);
		var title = strbundle.getString("inSetupPrefsTitle");
		prompts.alert(null, title, msg);
	}

	if( setup_res == null )
		return false;
	
	var failed_modules = "";
	var installed_modules = "";
	for( var i = 0; i < setup_res.length; ++i ) {
		if( (setup_res[i].status < 0) && (setup_res[i].status != -51) )
			result = false;
		
		if( gui && (setup_res[i].status == -100) ) {
			failed_modules += setup_res[i].name + " (" + setup_res[i].path + ")\n";
			result = false;
		}
		
		if( gui && result )
			installed_modules += setup_res[i].name + " (" + setup_res[i].path + ")\n";
	}
	if( failed_modules != "" ) {
		var msg = strbundle.getFormattedString("addModulesError", [failed_modules]);
		var title = strbundle.getString("inSetupPrefsTitle");
		prompts.alert(null, title, msg);
	}
	else if( gui ) {
		var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                        .getService(Components.interfaces.nsIPromptService);
		var strbundle = document.getElementById("strings");
		var title = strbundle.getString("inSetupPrefsTitle");
		var msg = strbundle.getFormattedString("addModulesSuccess", [installed_modules]);
		prompts.alert(null, title, msg);
	}
	return result;
}

function setup_is_needed( installed_modules_names )
{
	var obj = new PKCS11Setup();
	if( obj.isOK )
	{
		var arr = obj.search_for_pkcs11_module();
		if( arr )
		{
			var arrNames = new Array();
			var arrCounter = 0;
			var res = false;
			var name = {};
			for( var i = 0; i < arr.length; ++i )
			{
				if( !arr[i].isRegistred(name) )
					res = true;
				arrNames[arrCounter++] = name.value;
			}
			if( installed_modules_names )
				installed_modules_names.value = arrNames.join(", ");
			return res;
		}
	}
	return false;
}
